"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
const traverse_1 = __importDefault(require("@babel/traverse"));
const t = __importStar(require("@babel/types"));
const transformers_1 = require("./transformers");
const utils_1 = require("../utils");
const jsx_1 = require("./transformers/jsx");
const scope_1 = require("../scope");
const loops_1 = require("./transformers/loops");
const tertiary_1 = require("./transformers/tertiary");
const parse = (ast, clean = false) => {
    const result = clean ? JSON.parse(JSON.stringify(ast)) : ast;
    const scope = new scope_1.ParserScope();
    // validations and loading state definitions
    (0, traverse_1.default)(result, {
        ImportDeclaration(path) {
            if (path.node.source.value === 'react') {
                (0, utils_1.throwError)('cannot import react!');
            }
        },
        FunctionDeclaration(path) {
            // update cursor
            (0, transformers_1.manageCursorFunctionDeclaration)(path, scope);
        },
        VariableDeclaration(path) {
            // update cursor
            (0, transformers_1.manageCursorArrowFunctionExpressionVariableDeclaration)(path, scope);
        },
        JSXElement(path) {
            // skip first
            (0, transformers_1.skipSvgElements)(path);
            // get loopy
            // TODO need to support recursion
            // e.g. For in For in Show in For in Show
            // e.g. we need to return true if we find a For/Show and while loop sub traverse 
            (0, loops_1.parseLoops)(path, scope);
        },
        JSXAttribute(path) {
            (0, transformers_1.getForwardRefs)(path, scope);
        }
    });
    // teriary
    (0, traverse_1.default)(result, {
        FunctionDeclaration(path) {
            // update cursor
            (0, transformers_1.manageCursorFunctionDeclaration)(path, scope);
        },
        VariableDeclaration(path) {
            // update cursor
            (0, transformers_1.manageCursorArrowFunctionExpressionVariableDeclaration)(path, scope);
        },
        JSXElement(path) {
            // skip first
            (0, transformers_1.skipSvgElements)(path);
            // get loopy
            // TODO need to support recursion
            // e.g. For in For in Show in For in Show
            // e.g. we need to return true if we find a For/Show and while loop sub traverse 
            (0, tertiary_1.parseTertiary)(path, scope);
        }
    });
    // get options
    (0, traverse_1.default)(result, {
        ExpressionStatement(path) {
            (0, transformers_1.getConfigOptions)(path, scope);
        },
        JSXElement(path) {
            (0, transformers_1.skipSvgElements)(path);
        }
    });
    // now traverse :) 
    (0, traverse_1.default)(result, {
        ImportDeclaration(path) {
            // useRef, useStore, useState, useEffect, userDefaultProps
            (0, transformers_1.stubsToReact)(path);
        },
        FunctionDeclaration(path) {
            // update cursor
            (0, transformers_1.manageCursorFunctionDeclaration)(path, scope);
        },
        VariableDeclaration(path) {
            // // update cursor
            (0, transformers_1.manageCursorArrowFunctionExpressionVariableDeclaration)(path, scope);
        },
        VariableDeclarator(path) {
            // useRef transformer
            (0, transformers_1.useRefVariableDeclarator)(path, scope);
            // useState transformer
            (0, transformers_1.useStateVariableDeclarator)(path, scope);
            // useStore transformer
            (0, transformers_1.useStoreVariableDeclarator)(path, scope);
        },
        AssignmentExpression(path) {
            // STATE
            // FROM state.property = value;
            // TO   setProperty(value);
            (0, transformers_1.stateAssignments)(path, scope);
        },
        CallExpression(path) {
            // state.hide(el);
            (0, transformers_1.stateFunctions)(path, scope);
            // useDefault props transformer
            (0, transformers_1.defaultProps)(path, scope);
        },
        ExpressionStatement(path) {
            // onMount, onUnMount, onUpdate
            (0, transformers_1.lifecycleHooks)(path);
        },
        JSXElement(path) {
            // always ensure to use parents when returning JSX
            (0, jsx_1.jsxWithParens)(path);
            (0, transformers_1.skipSvgElements)(path);
        },
        // TODO set configOptions for strings
        StringLiteral(path) {
            // @ts-ignore
            if (path.node.extra && path.node.extra.raw.startsWith('"')) {
                path.node.extra.raw = `'${path.node.value.replace(/'/g, "\\'")}'`;
            }
        }
    });
    // update all the state!
    // update all the refs!
    // move default props!
    (0, traverse_1.default)(result, {
        MemberExpression(path) {
            if (path.node.object.type === 'Identifier' &&
                path.node.object.name === 'state') {
                path.replaceWith(path.node.property);
            }
        },
        JSXElement(path) {
            (0, transformers_1.skipSvgElements)(path);
        }
    });
    (0, traverse_1.default)(result, {
        FunctionDeclaration(path) {
            // update cursor
            (0, transformers_1.manageCursorFunctionDeclaration)(path, scope);
        },
        VariableDeclaration(path) {
            // update cursor
            (0, transformers_1.manageCursorArrowFunctionExpressionVariableDeclaration)(path, scope);
        },
        Identifier(path) {
            (0, transformers_1.updateRefsIdentifier)(path, scope);
        },
        JSXElement(path) {
            (0, transformers_1.skipSvgElements)(path);
        },
        JSXAttribute(path) {
            (0, transformers_1.processForwardRefs)(path, scope);
        }
    });
    // top-level forward refs
    (0, traverse_1.default)(result, {
        FunctionDeclaration(path) {
            // update cursor
            (0, transformers_1.manageCursorFunctionDeclaration)(path, scope);
            // update ref methods
            (0, transformers_1.updateRefFunctionDeclarations)(path, scope);
        },
        VariableDeclaration(path) {
            // update cursor
            (0, transformers_1.manageCursorArrowFunctionExpressionVariableDeclaration)(path, scope);
        },
        JSXElement(path) {
            (0, transformers_1.skipSvgElements)(path);
        }
    });
    if (scope.configOptions?.rsc?.componentType === 'client') {
        result.program.directives = [
            t.directive(t.directiveLiteral('use client'))
        ];
    }
    const imports = [];
    const exports = [];
    // TODO useMetadata options to set
    imports.push(t.importDeclaration([t.importNamespaceSpecifier(t.identifier('React'))], t.stringLiteral('react')));
    // add forwardRef imports
    if (scope.components.some(component => component.hasForwardRef())) {
        imports.push(t.importDeclaration([
            t.importSpecifier(t.identifier('forwardRef'), t.identifier('forwardRef'))
        ], t.stringLiteral('react')));
        if (scope.defaultForwardRefExport) {
            exports.push(t.exportDefaultDeclaration(t.identifier(scope.defaultForwardRefExport)));
        }
    }
    result.program.body = [
        ...imports,
        ...result.program.body,
        ...exports
    ];
    return {
        scope,
        ast: result
    };
};
exports.parse = parse;
