import * as t from '@babel/types';
import { throwError } from '../utils';
export const updateRefsIdentifier = (path, { componentCursor, componentHash }) => {
    // in a MemberExpression?
    if (path.parent.type === 'MemberExpression') {
        return;
    }
    // in a VariableDeclarator?
    if (path.parent.type === 'VariableDeclarator') {
        return;
    }
    // is a ref the same name?
    if (!componentHash[componentCursor].refs.hasOwnProperty(path.node.name)) {
        return;
    }
    path.replaceWith(t.memberExpression(t.identifier(path.node.name), t.identifier('current')));
};
export const useRefVariableDeclarator = (path, { componentCursor, componentHash }) => {
    if (path.node.init &&
        path.node.init.type === 'CallExpression' &&
        path.node.init.callee &&
        path.node.init.callee.type === 'Identifier' &&
        path.node.init.callee.name === 'useRef') {
        const node = path.node;
        if (node.id.type !== 'Identifier') {
            return throwError('useRef must use an Identifier');
        }
        if (componentHash[componentCursor].refs[node.id.name] &&
            componentHash[componentCursor].refs[node.id.name].hasOwnProperty('type')) {
            // console.log(node.id)
            // throwError('refs variable is already defined!')
        }
        else {
            // TODO fix if case above so it can throw
            componentHash[componentCursor].refs[node.id.name] = {
                orig: 'useRef',
                type: 'ObjectProperty',
                used: false
            };
        }
    }
};
