import { throwError } from "../utils";
export class ParserComponentRef {
    name;
    used;
    forward;
    constructor(options) {
        this.name = options.name;
        this.used = options.used ?? false;
        this.forward = options.forward ?? false;
    }
}
class ParserComponentStateProperty {
    name;
    derived;
    from;
    used;
    constructor(options) {
        this.name = options.name;
        this.derived = options.derived;
        this.from = options.from;
        this.used = options.used ?? false;
    }
    setUsed(used) {
        this.used = used;
    }
}
class ParserComponentState {
    properties;
    constructor() {
        this.properties = [];
    }
    exists(name) {
        return this.properties.some(property => property.name === name);
    }
    get(name) {
        return this.properties.find(property => property.name === name);
    }
}
class ParserComponent {
    name;
    refs;
    state;
    constructor(name) {
        this.name = name;
        this.refs = [];
        this.state = new ParserComponentState();
    }
    addRef(options) {
        const ref = new ParserComponentRef(options);
        this.refs.push(ref);
    }
    findRef(name) {
        return this.refs.find(ref => ref.name === name);
    }
    hasRef(name) {
        return this.refs.some(ref => ref.name === name);
    }
    addProp(options) {
        const property = new ParserComponentStateProperty(options);
        this.state.properties.push(property);
    }
    findProp(name) {
        return this.state.get(name);
    }
    hasProp(name) {
        return this.state.exists(name);
    }
}
export class ParserScope {
    components;
    cursor;
    constructor() {
        this.components = [];
    }
    addComponent(name) {
        const existingComponent = this.findComponent(name);
        if (!existingComponent) {
            const component = new ParserComponent(name);
            this.components.push(component);
        }
    }
    findComponent(name) {
        return this.components.find(component => component.name === name);
    }
    currentComponent(throws = false) {
        if (!this.cursor) {
            throwError(this.errorMsg('COMP_NOT_FOUND'));
        }
        if (this.cursor) {
            return this.findComponent(this.cursor);
        }
        return undefined;
    }
    setCursor(newCursor) {
        if (newCursor !== null && newCursor !== undefined) {
            this.cursor = newCursor;
        }
    }
    errorMsg(error, params) {
        switch (error) {
            case 'COMP_NOT_FOUND':
                return `Current Component not found!`;
            case 'REF_NOT_FOUND':
                return `ref does not exist Component: ${this.cursor} Ref: ${params.ref}!`;
            case 'PROP_FUNC_NOT_FOUND':
                return `looking up state function on non-existent Component with Prop: ${params.prop}`;
            case 'PROP_FUNC_COMP_NOT_FOUND':
                return `looking up state function on non-existent prop Component: ${this.cursor} Prop: ${params.prop}`;
            case 'PROP_NOT_FOUND':
                return `looking up state property on non-existent Component with Prop: ${params.prop}`;
            case 'PROP_SET_NOT_FOUND':
                return `current component state set without being defined: Component: ${this.cursor} prop: ${params.prop}`;
        }
    }
}
