import * as t from '@babel/types';
import traverse, { NodePath } from '@babel/traverse';
import { ParserScope } from '../../scope';
import { throwError } from '../../utils';

export const parseLoops = (
    path: NodePath<t.JSXElement>,
    scope: ParserScope
) => {
    if (path.node.openingElement.type !== 'JSXOpeningElement') return;
    if (path.node.openingElement.name.type !== 'JSXIdentifier') return;
    if (path.node.openingElement.name.name !== 'For') return;
    if (path.node.openingElement.attributes[0].type !== 'JSXAttribute') return;
    const el: t.JSXAttribute = path.node.openingElement.attributes[0];
    if (el.name.type !== 'JSXIdentifier') return;
    if (el.name.name !== 'each') {
        throwError('For requires each prop!');
    }
    const exprContainer: t.JSXExpressionContainer = el.value as t.JSXExpressionContainer;
    /////
    /////
    /////

    const innerExprContainer: t.JSXExpressionContainer = path.node.children.find(n => n.type === 'JSXExpressionContainer') as any;

    // RECURSION
    // base recursion case
    if (!innerExprContainer) return;

    if (innerExprContainer.type !== 'JSXExpressionContainer') {
        throwError('For requires JSXExpressionContainer!');
    }
    const innerExpr: t.ArrowFunctionExpression = innerExprContainer.expression as t.ArrowFunctionExpression;
    if (innerExpr.params.length !== 2) {
        throwError('For requires 2 params!');
    }
    const item: t.Identifier = innerExpr.params[0] as t.Identifier;
    const index: t.Identifier = innerExpr.params[1] as t.Identifier;

    const body = innerExpr.body;
    const expr = t.optionalCallExpression(
        t.optionalMemberExpression(
            exprContainer.expression as any,
            t.identifier('map'),
            false,
            true
        ),
        [
            t.arrowFunctionExpression(
                [
                    item,
                    index
                ],
                body

            )
        ],
        false
    );

    return path.replaceWith(t.jsxExpressionContainer(expr));

    // RECURSION
    // RECURSION
    // RECURSION


    const programNode = t.file(
        t.program([
            t.functionDeclaration(
                t.identifier('fakeFunction'),
                [],
                t.blockStatement([
                    t.returnStatement(
                        // RECURSION
                        // JSX goes here
                        expr
                    )
                ])
            )
        ])
    );

    // bake
    traverse(programNode, {
        JSXElement(returnPath) {
            parseLoops(returnPath, scope)
        }
    });

    // pull out of the oven
    const funcDecl: t.FunctionDeclaration = programNode.program.body[0] as t.FunctionDeclaration;
    const blockStm: t.BlockStatement = funcDecl.body as t.BlockStatement;
    const retnStmt: t.ReturnStatement = blockStm.body[0] as any;

    const output = t.jsxExpressionContainer(
        retnStmt.argument
    );
    path.replaceWith(
        output
    );

};