import * as t from '@babel/types';
import traverse, { NodePath } from '@babel/traverse';
import { ParserScope } from '../../scope';
import { throwError } from '../../utils';
import stringify from 'ast-stringify';

export const parseTertiary = (
    path: NodePath<t.JSXElement>,
    scope: ParserScope
) => {
    if (path.node.openingElement.type !== 'JSXOpeningElement') return;
    if (path.node.openingElement.name.type !== 'JSXIdentifier') return;
    if (path.node.openingElement.name.name !== 'Show') return;
    if (path.node.openingElement.attributes.length !== 2) {
        throwError('Show requires 2 arguments: when, fallback');
    }
    if (path.node.openingElement.attributes[0].type !== 'JSXAttribute') return;
    if (path.node.openingElement.attributes[1].type !== 'JSXAttribute') return;

    const when: t.JSXAttribute = path.node.openingElement.attributes[0];
    if (when.name.type !== 'JSXIdentifier') return;
    if (when.name.name !== 'when') {
        throwError('Show requires when prop!');
    }
    const whenExprContainer: t.JSXExpressionContainer = when.value as t.JSXExpressionContainer;

    const fallback: t.JSXAttribute = path.node.openingElement.attributes[1];
    if (fallback.name.type !== 'JSXIdentifier') return;
    if (fallback.name.name !== 'fallback') {
        throwError('Show requires fallback prop!');
    }
    const fallbackExprContainer: t.JSXExpressionContainer = fallback.value as t.JSXExpressionContainer;


    const jsxEl: t.JSXElement = path.node.children.find(el => el.type === 'JSXElement') as t.JSXElement;

    // RECURSION
    // base recursion case
    if (!jsxEl) return;


    const finalExpr =
        t.conditionalExpression(
            whenExprContainer.expression as t.Expression,
            jsxEl,
            fallbackExprContainer.expression as t.Expression
        );


    return path.replaceWith(t.jsxExpressionContainer(finalExpr));
    // path.replaceWith(t.jsxExpressionContainer(finalExpr));
    // return true;


    // RECURSION
    // RECURSION
    // RECURSION

    const programNode = t.file(
        t.program([
            t.functionDeclaration(
                t.identifier('fakeFunction'),
                [],
                t.blockStatement([
                    t.returnStatement(
                        // RECURSION
                        // JSX goes here
                        finalExpr as any
                    )
                ])
            )
        ])
    );

    // bake
    traverse(programNode, {
        JSXElement(returnPath) {
            parseTertiary(returnPath, scope)
        }
    });

    // pull out of the oven
    const funcDecl: t.FunctionDeclaration = programNode.program.body[0] as t.FunctionDeclaration;
    const blockStm: t.BlockStatement = funcDecl.body as t.BlockStatement;
    const retnStmt: t.ReturnStatement = blockStm.body[0] as any;

    const output = t.jsxExpressionContainer(
        retnStmt.argument
    );
    path.replaceWith(
        output
    );
};